(*----------------------------------------------------------------------*)
(*                Set_Params --- Set Communications Parameters          *)
(*----------------------------------------------------------------------*)

OVERLAY FUNCTION Set_Params( First_Time : BOOLEAN;
                             Use_Script : BOOLEAN ) : BOOLEAN;

(*----------------------------------------------------------------------*)
(*                                                                      *)
(*     Function:  Set_Params                                            *)
(*                                                                      *)
(*     Purpose:    Set communications parameters                        *)
(*                                                                      *)
(*     Calling Sequence:                                                *)
(*                                                                      *)
(*        Flag := Set_Params( First_Time : BOOLEAN;                     *)
(*                            Use_Script : BOOLEAN ) : BOOLEAN;         *)
(*                                                                      *)
(*           First_Time:  TRUE for initial setup, else FALSE.           *)
(*           Use_Script:  TRUE to use commands from script buffer       *)
(*                                                                      *)
(*           Flag is TRUE if successful set-up, else FALSE.             *)
(*                                                                      *)
(*      Calls:   ClrScr                                                 *)
(*               Async_Init                                             *)
(*               Async_Open                                             *)
(*               Get_Params                                             *)
(*                                                                      *)
(*----------------------------------------------------------------------*)

VAR                                (* Hold new colors/video mode *)
   New_Text_Mode        : INTEGER;
   New_ForeGround_Color : INTEGER;
   New_BackGround_Color : INTEGER;
   New_Menu_Text_Color  : INTEGER;
   New_Menu_Frame_Color : INTEGER;
   Review_Buffer_Length : INTEGER;

CONST
   Max_Param_Names = 63;

(* STRUCTURED *) CONST
   Param_Names : ARRAY[ 1 .. Max_Param_Names ] OF STRING[2] =
                 ( 'DA', 'PA', 'ST', 'BA', 'PO', 'BS', 'DE', 'TE', 'AG',
                   'LF', 'MI', 'MD', 'MC', 'MN', 'MB', 'ME', 'MT', 'MH',
                   'MO', 'MR', 'SM', 'PM', 'LE', 'FC', 'FD', 'FW', 'TT',
                   'SD', 'GD', 'TB', 'MA', 'MS', 'FM', 'TM', 'CF', 'CB',
                   'CT', 'CM', 'MW', 'BP', 'EB', 'DP', 'BB', 'XM', 'CH',
                   'XB', 'AB', 'VB', 'VF', 'VE', 'VU', 'KC', 'KD', 'KE',
                   'KH', 'KN', 'KP', 'KQ', 'KR', 'KS', 'KT', 'KW', 'K8' );

   Trans_Type_Name : ARRAY[ Transfer_Type ] OF STRING[2] =
                     ( 'AS', 'XK', 'XC', 'KE', 'TE', 'MK',
                       'M7', 'YM', 'YB', 'NO' );

   Trans_Type      : ARRAY[ 1 .. Max_Transfer_Types ] OF Transfer_Type =
                     ( Ascii, Xmodem_Chk, Xmodem_Crc, Kermit, Telink,
                       Modem7_Chk, Modem7_CRC, Ymodem, Ymodem_Batch, None );

(*----------------------------------------------------------------------*)
(*    Write_Config_File --- Write updated PibTerm configuration file    *)
(*----------------------------------------------------------------------*)

PROCEDURE Write_Config_File;

(*----------------------------------------------------------------------*)
(*                                                                      *)
(*     Procedure:  Write_Config_File                                    *)
(*                                                                      *)
(*     Purpose:    Writes updated parameters to PibTerm config. file    *)
(*                                                                      *)
(*     Calling Sequence:                                                *)
(*                                                                      *)
(*        Write_Config_File;                                            *)
(*                                                                      *)
(*      Calls:   None                                                   *)
(*                                                                      *)
(*----------------------------------------------------------------------*)

BEGIN (* Write_Config_File *)

   ASSIGN( Config_File, Home_Dir + 'PIBTERM.CNF' );
             (*$I-*)
   REWRITE( Config_File );
             (*$I+*)

   IF Int24Result <> 0 THEN
      BEGIN
         WRITELN;
         WRITELN('Can''t create new PIBTERM.CNF.');
      END
   ELSE
      BEGIN  (* Write configuration file *)

         WRITELN( Config_File , 'FC=', FK_CR                         );
         WRITELN( Config_File , 'FD=', FK_Delay                      );
         WRITELN( Config_File , 'FW=', FK_Wait_For                   );
         WRITELN( Config_File , 'FM=', FK_Ctrl_Mark                  );
         WRITELN( ConFig_File , 'DA=', Data_Bits                     );
         WRITELN( ConFig_File , 'PA=', Parity                        );
         WRITELN( Config_File , 'ST=', Stop_Bits                     );
         WRITELN( Config_File , 'BA=', Baud_Rate                     );
         WRITELN( Config_File , 'PO=', Comm_Port                     );
         WRITELN( Config_File , 'BS=', ORD( BS_Char )                );
         WRITELN( Config_File , 'DE=', ORD( Ctrl_BS_Char )           );
         WRITELN( Config_File , 'TE=', ORD( Terminal_To_Emulate )    );
         WRITELN( Config_File , 'AG=', ORD( Ansi_Graphics_On )       );
         WRITELN( Config_File , 'LF=', ORD( Add_LF )                 );
         WRITELN( Config_File , 'MI=', Write_Ctrls( Modem_Init )     );
         WRITELN( Config_File , 'MD=', Write_Ctrls( Modem_Dial )     );
         WRITELN( Config_File , 'MC=', Modem_Connect                 );
         WRITELN( Config_File , 'MN=', Modem_No_Carrier              );
         WRITELN( Config_File , 'MB=', Modem_Busy                    );
         WRITELN( Config_File , 'ME=', Write_Ctrls( Modem_Escape)    );
         WRITELN( Config_File , 'MT=', Modem_Escape_Time             );
         WRITELN( Config_File , 'MH=', Write_Ctrls( Modem_Hang_Up)   );
         WRITELN( Config_File , 'MO=', Modem_Time_Out:4:0            );
         WRITELN( Config_File , 'MR=', Modem_Redial_Delay:4:0        );
         WRITELN( Config_File , 'MA=', Write_Ctrls( Modem_Answer )   );
         WRITELN( Config_File , 'MS=', Write_Ctrls( Modem_Host_Set ) );
         WRITELN( Config_File , 'MW=', Modem_Command_Delay           );
         WRITELN( Config_File , 'SM=', ORD( Silent_Mode )            );
         WRITELN( Config_File , 'PM=', ORD( Play_Music_On )          );
         WRITELN( Config_File , 'LE=', ORD( Local_Echo )             );
         WRITELN( Config_File , 'TT=', Trans_Type_Name[ Default_Transfer_Type ] );
         WRITELN( Config_File , 'SD=', Screen_Dump_Name              );
         WRITELN( Config_File , 'GD=', GMT_Difference                );
         WRITELN( Config_File , 'TB=', Transfer_Bells                );

         IF New_Text_Mode = BW80 THEN
            WRITELN( Config_File , 'TM=M' )
         ELSE
            WRITELN( Config_File , 'TM=C' );

         WRITELN( Config_File , 'CF=', New_ForeGround_Color          );
         WRITELN( Config_File , 'CB=', New_BackGround_Color          );
         WRITELN( Config_File , 'CT=', New_Menu_Text_Color           );
         WRITELN( Config_File , 'CM=', New_Menu_Frame_Color          );
         WRITELN( Config_File , 'BP=', ORD( CompuServe_B_On )        );
         WRITELN( Config_File , 'EB=', ORD( Mahoney_On )             );
         WRITELN( Config_File , 'DP=', Default_Prefix                );
         WRITELN( Config_File , 'BB=', Max_Review_Length             );
         WRITELN( Config_File , 'XM=', ORD( Exploding_Menus )        );
         WRITELN( Config_File , 'CH=', ORD( Modem_Carrier_High )     );
         WRITELN( Config_File , 'XB=', Max_Write_Buffer              );
         WRITELN( Config_File , 'AB=', ORD( Host_Auto_Baud )         );
         WRITELN( Config_File , 'VB=', VT100_BackGround_Color        );
         WRITELN( Config_File , 'VF=', VT100_ForeGround_Color        );
         WRITELN( Config_File , 'VE=', VT100_Bold_Color              );
         WRITELN( Config_File , 'VU=', VT100_Underline_Color         );
         WRITELN( Config_File , 'KC=', Kermit_Chk_Type               );
         WRITELN( Config_File , 'KD=', ORD( Kermit_Debug )           );
         WRITELN( Config_File , 'KE=', Write_Ctrls( Kermit_EOL )     );
         WRITELN( Config_File , 'KH=', Write_Ctrls( Kermit_Header_Char ) );
         WRITELN( Config_File , 'KN=', Kermit_Npad                   );
         WRITELN( Config_File , 'KP=', Write_Ctrls( Kermit_Pad_Char ) );
         WRITELN( Config_File , 'KQ=', Write_Ctrls( Kermit_Quote_Char ) );
         WRITELN( Config_File , 'KR=', Write_Ctrls( Kermit_Repeat_Char ) );
         WRITELN( Config_File , 'KS=', Kermit_Packet_Size            );
         WRITELN( Config_File , 'KT=', Kermit_TimeOut                );
         WRITELN( Config_File , 'KW=', Kermit_Delay_Time             );
         WRITELN( Config_File , 'K8=', Write_Ctrls( Kermit_Quote_8_Char ) );

   END   (* Write configuration file *);

   CLOSE( Config_File );

END   (* Write_Config_File *);

(*----------------------------------------------------------------------*)
(*         Set_Parameter --- Set value of PibTerm parameter             *)
(*----------------------------------------------------------------------*)

PROCEDURE Set_Parameter( Param_Num  : INTEGER;
                         Param_Ival : INTEGER;
                         Param_Rval : REAL;
                         Param_Str  : AnyStr   );

(*----------------------------------------------------------------------*)
(*                                                                      *)
(*     Procedure:  Set_Parameter                                        *)
(*                                                                      *)
(*     Purpose:    Set value of PibTerm parameter                       *)
(*                                                                      *)
(*     Calling Sequence:                                                *)
(*                                                                      *)
(*        Set_Parameter( Param_Num  : INTEGER;                          *)
(*                       Param_Ival : INTEGER;                          *)
(*                       Param_Rval : REAL;                             *)
(*                       Param_Str  : AnyStr   );                       *)
(*                                                                      *)
(*           Param_Num  --- Parameter to set                            *)
(*           Param_Ival --- integer parameter value                     *)
(*           Param_Rval --- real parameter value                        *)
(*           Param_Str  --- string parameter value                      *)
(*                                                                      *)
(*----------------------------------------------------------------------*)

VAR
   I : INTEGER;

BEGIN (* Set_Parameter *)

   CASE Param_Num OF

       0 :  ;

       1 :  Data_Bits     := Param_Ival;
       2 :  Parity        := Param_Str[1];
       3 :  Stop_Bits     := Param_Ival;
       4 :  Baud_Rate     := Param_Ival;
       5 :  Comm_Port     := Param_Ival;
       6 :  BS_Char       := CHR( Param_Ival );
       7 :  Ctrl_BS_Char  := CHR( Param_Ival );
       8 :  CASE Param_Ival OF
               0:     Terminal_To_Emulate := Dumb;
               1:     Terminal_To_Emulate := VT52;
               2:     Terminal_To_Emulate := Ansi;
               3:     Terminal_To_Emulate := VT100;
               4:     Terminal_To_Emulate := Gossip;
               5:     Terminal_To_Emulate := HostMode;
               ELSE   Terminal_To_Emulate := Dumb;
            END (* CASE *);
       9 :  Ansi_Graphics_On   := ( Param_Ival = 1 );
      10 :  Add_Lf             := ( Param_Ival = 1 );
      11 :  Modem_Init         := Read_Ctrls( Param_Str );
      12 :  Modem_Dial         := Read_Ctrls( Param_Str );
      13 :  Modem_Connect      := Param_Str;
      14 :  Modem_No_Carrier   := Param_Str;
      15 :  Modem_Busy         := Param_Str;
      16 :  Modem_Escape       := Read_Ctrls( Param_Str );
      17 :  Modem_Escape_Time  := Param_Ival;
      18 :  Modem_Hang_Up      := Read_Ctrls( Param_Str );
      19 :  Modem_Time_Out     := Param_Rval;
      20 :  Modem_Redial_Delay := Param_Rval;
      21 :  Silent_Mode        := ( Param_Ival = 1 );
      22 :  Play_Music_On      := ( Param_Ival = 1 );
      23 :  Local_Echo         := ( Param_Ival = 1 );
      24 :  FK_CR              := Param_Str[1];
      25 :  FK_Delay           := Param_Str[1];
      26 :  FK_Wait_For        := Param_Str[1];
      27 :  FOR I := 1 TO Max_Transfer_Types DO
               IF ( COPY( Param_Str, 1, 2 ) =
                  Trans_Type_Name[Trans_Type[I]] ) THEN
                     Default_Transfer_Type := Trans_Type[I];
      28 :  Screen_Dump_Name   := Param_Str;
      29 :  GMT_Difference     := Param_Ival;
      30 :  Transfer_Bells     := Param_Ival;
      31 :  Modem_Answer       := Read_Ctrls( Param_Str );
      32 :  Modem_Host_Set     := Read_Ctrls( Param_Str );
      33 :  FK_Ctrl_Mark       := Param_Str[1];
      34 :  CASE Param_Str[1] OF
               'C': New_Text_Mode   := C80;
               'M': New_Text_Mode   := BW80;
               ELSE New_Text_Mode   := BW80;
            END;
      35 :  New_ForeGround_Color   := Param_Ival;
      36 :  New_BackGround_Color   := Param_Ival;
      37 :  New_Menu_Text_Color    := Param_Ival;
      38 :  New_Menu_Frame_Color   := Param_Ival;
      39 :  Modem_Command_Delay    := Param_Ival;
      40 :  CompuServe_B_On        := ( Param_Ival = 1 );
      41 :  Mahoney_On             := ( Param_Ival = 1 );
      42 :  Default_Prefix         := Param_Str[1];
      43 :  Max_Review_Length      := MIN( MAX( Param_Ival , 0 ) , 800 );
      44 :  Exploding_Menus        := ( Param_Ival = 1 );
      45 :  Modem_Carrier_High     := ( Param_Ival = 1 );
      46 :  Max_Write_Buffer       := MAX( ( ( Param_Ival + 127 ) DIV 128 ) * 128 ,
                                           128 );
      47 :  Host_Auto_Baud         := ( Param_Ival = 1 );
      48 :  VT100_BackGround_Color := Param_Ival;
      49 :  VT100_ForeGround_Color := Param_Ival;
      50 :  VT100_Bold_Color       := Param_Ival;
      51 :  VT100_Underline_Color  := Param_Ival;
      52 :  Kermit_Chk_Type        := Param_Str[1];
      53 :  Kermit_Debug           := ( Param_Ival = 1 );
      54 :  Kermit_EOL             := COPY( Read_Ctrls( Param_Str ), 1, 1 );
      55 :  Kermit_Header_Char     := COPY( Read_Ctrls( Param_Str ), 1, 1 ) ;
      56 :  Kermit_Npad            := MAX( Param_Ival , 0 );
      57 :  Kermit_Pad_Char        := COPY( Read_Ctrls( Param_Str ), 1, 1 );
      58 :  Kermit_Quote_Char      := COPY( Read_Ctrls( Param_Str ), 1, 1 );
      59 :  Kermit_Repeat_Char     := COPY( Read_Ctrls( Param_Str ), 1, 1 );
      60 :  Kermit_Packet_Size     := MIN( MAX( Param_Ival , 20 ) , 94 );
      61 :  Kermit_TimeOut         := Param_Ival;
      62 :  Kermit_Delay_Time      := Param_Ival;
      63 :  Kermit_Quote_8_Char    := COPY( Read_Ctrls( Param_Str ), 1, 1 );

      ELSE ;

   END (* CASE *);

   IF Silent_Mode THEN Play_Music_On := FALSE;
   Menu_Set_Explode( Exploding_Menus );

END   (* Set_Parameter *);

(*----------------------------------------------------------------------*)
(*        Set_Defaults --- Set Default Communications Parameters        *)
(*----------------------------------------------------------------------*)

OVERLAY PROCEDURE Set_Defaults;

(*----------------------------------------------------------------------*)
(*                                                                      *)
(*     Procedure:  Set_Defaults                                         *)
(*                                                                      *)
(*     Purpose:    Set default communications parameters                *)
(*                                                                      *)
(*     Calling Sequence:                                                *)
(*                                                                      *)
(*        Set_Defaults;                                                 *)
(*                                                                      *)
(*      Calls:   None                                                   *)
(*                                                                      *)
(*      Remarks:  Note that defaults for color-related variables are    *)
(*                set in PibTerm so that error messages will appear     *)
(*                in a readable form.                                   *)
(*                                                                      *)
(*----------------------------------------------------------------------*)

VAR
   I: INTEGER;
   J: INTEGER;

BEGIN (* Set_Defaults *)

   Data_Bits           := 8;
   Parity              := 'N';
   Stop_Bits           := 1;
   Baud_Rate           := 2400;
   Comm_Port           := 1;

   BS_Char             := CHR( BS );
   Ctrl_BS_Char        := CHR( DEL );

   Terminal_To_Emulate := Dumb;

   Ansi_Graphics_On    := FALSE;
   Add_LF              := FALSE;
   Capture_On          := FALSE;
   Printer_On          := FALSE;
   Translate_On        := FALSE;
   Play_Music_On       := TRUE;
   Silent_Mode         := FALSE;
   Local_Echo          := FALSE;

   Modem_Init          := 'ATX3|';
   Modem_Dial          := 'ATDT';
   Modem_Connect       := 'CONNECT';
   Modem_No_Carrier    := 'NO CARRIER';
   Modem_Busy          := 'BUSY';
   Modem_Time_Out      := 60.0;
   Modem_Redial_Delay  := 45.0;
   Modem_Escape        := '+++';
   Modem_Escape_Time   := 1500;
   Modem_Hang_Up       := 'ATH0|';
   Modem_Answer        := 'ATA|';
   Modem_Host_Set      := 'ATZ|~ATX1|~ATS0=1|';
   Modem_Command_Delay := 0;

   Host_Auto_Baud      := TRUE;

   Script_String       := '';
   Script_String_2     := '';

   Max_Write_Buffer    := 2048;

   Default_Transfer_Type := Xmodem_Chk;
   Default_Prefix        := '+';

   Phone_Entry_Page    := 1;

   FOR I := 1 TO 4 DO
      FOR J := 1 TO 10 DO
         Function_Keys[I,J] := '';

   FK_CR        := '|';
   FK_Delay     := '~';
   FK_Wait_For  := '`';
   FK_Ctrl_Mark := '^';

   FOR I := 0 TO 255 DO
      BEGIN
         TrTab[ CHR( I ) ]     := CHR( I );
         Keypad_Key_Index[ I ] := 0;
      END;

   Screen_Dump_Name := '';

   GMT_Difference   := 0;
   Transfer_Bells   := 8;

   CompuServe_B_On   := FALSE;
   Mahoney_On        := TRUE;
   Exploding_Menus   := FALSE;

   Review_On         := FALSE;
   Max_Review_Length := 0;
   Review_Buffer     := NIL;
   Review_Head       := 0;
   Review_Tail       := 0;
   Review_Line       := '';

   FOR I := 1 TO 3 DO
      FOR J := 1 TO 10 DO
         Keypad_Keys[I,J] := '';

   Keypad_Key_Index[U_Arrow]      := 1;
   Keypad_Key_Index[Alt_U_Arrow]  := 11;
   Keypad_Key_Index[Ctrl_U_Arrow] := 21;

   Keypad_Key_Index[L_Arrow]      := 2;
   Keypad_Key_Index[Alt_L_Arrow]  := 12;
   Keypad_Key_Index[Ctrl_L_Arrow] := 22;

   Keypad_Key_Index[R_Arrow]      := 3;
   Keypad_Key_Index[Alt_R_Arrow]  := 13;
   Keypad_Key_Index[Ctrl_R_Arrow] := 23;

   Keypad_Key_Index[D_Arrow]      := 4;
   Keypad_Key_Index[Alt_D_Arrow]  := 14;
   Keypad_Key_Index[Ctrl_D_Arrow] := 24;

   Keypad_Key_Index[Home]         := 5;
   Keypad_Key_Index[Alt_Home]     := 15;

   Keypad_Key_Index[PgUp]         := 6;
   Keypad_Key_Index[Alt_PgUp]     := 16;
   Keypad_Key_Index[Ctrl_PgUp]    := 26;

   Keypad_Key_Index[End_Key]      := 7;
   Keypad_Key_Index[Alt_End_Key]  := 17;
   Keypad_Key_Index[Ctrl_End_Key] := 27;

   Keypad_Key_Index[PgDn]         := 8;
   Keypad_Key_Index[Alt_PgDn]     := 18;
   Keypad_Key_Index[Ctrl_PgDn]    := 28;

   Keypad_Key_Index[Ins_Key]      := 9;
   Keypad_Key_Index[Alt_Ins_Key]  := 19;
   Keypad_Key_Index[Ctrl_Ins_Key] := 29;

   Keypad_Key_Index[Del_Key]      := 10;
   Keypad_Key_Index[Alt_Del_Key]  := 20;
   Keypad_Key_Index[Ctrl_Del_Key] := 30;

   Keypad_Key_Names[1,1]  := 'K8';
   Keypad_Key_Names[2,1]  := 'AK8';
   Keypad_Key_Names[3,1]  := 'CK8';

   Keypad_Key_Names[1,2]  := 'K4';
   Keypad_Key_Names[2,2]  := 'AK4';
   Keypad_Key_Names[3,2]  := 'CK4';

   Keypad_Key_Names[1,3]  := 'K6';
   Keypad_Key_Names[2,3]  := 'AK6';
   Keypad_Key_Names[3,3]  := 'CK6';

   Keypad_Key_Names[1,4]  := 'K2';
   Keypad_Key_Names[2,4]  := 'AK2';
   Keypad_Key_Names[3,4]  := 'CK2';

   Keypad_Key_Names[1,5]  := 'K7';
   Keypad_Key_Names[2,5]  := 'AK7';
   Keypad_Key_Names[3,5]  := '';

   Keypad_Key_Names[1,6]  := 'K9';
   Keypad_Key_Names[2,6]  := 'AK9';
   Keypad_Key_Names[3,6]  := 'CK9';

   Keypad_Key_Names[1,7]  := 'K1';
   Keypad_Key_Names[2,7]  := 'AK1';
   Keypad_Key_Names[3,7]  := 'CK1';

   Keypad_Key_Names[1,8]  := 'K3';
   Keypad_Key_Names[2,8]  := 'AK3';
   Keypad_Key_Names[3,8]  := 'CK3';

   Keypad_Key_Names[1,9]  := 'K0';
   Keypad_Key_Names[2,9]  := 'AK0';
   Keypad_Key_Names[3,9]  := 'CK0';

   Keypad_Key_Names[1,10] := 'K.';
   Keypad_Key_Names[2,10] := 'AK.';
   Keypad_Key_Names[3,10] := 'CK.';

                                   (* Initialize command table *)
   FOR I := 0 TO 255 DO
      Pibterm_Command_Table[I] := Null_Command;

   PibTerm_Command_Table[Alt_A] := EditSy;
   PibTerm_Command_Table[Alt_B] := BreakSy;
   PibTerm_Command_Table[Alt_C] := ClearSy;
   PibTerm_Command_Table[Alt_D] := DialSy;
   PibTerm_Command_Table[Alt_E] := EchoSy;
   PibTerm_Command_Table[Alt_F] := FileSy;
   PibTerm_Command_Table[Alt_G] := GossipSy;
   PibTerm_Command_Table[Alt_H] := HangUpSy;
   PibTerm_Command_Table[Alt_I] := InfoSy;
   PibTerm_Command_Table[Alt_J] := DosSy;
   PibTerm_Command_Table[Alt_K] := KeySy;
   PibTerm_Command_Table[Alt_L] := LogSy;
   PibTerm_Command_Table[Alt_M] := MuteSy;
   PibTerm_Command_Table[Alt_N] := FastCSy;
   PibTerm_Command_Table[Alt_O] := CaptureSy;
   PibTerm_Command_Table[Alt_P] := ParamSy;
   PibTerm_Command_Table[Alt_Q] := RedialSy;
   PibTerm_Command_Table[Alt_R] := ReceiveSy;
   PibTerm_Command_Table[Alt_S] := SendSy;
   PibTerm_Command_Table[Alt_T] := TranslateSy;
   PibTerm_Command_Table[Alt_U] := SDumpSy;
   PibTerm_Command_Table[Alt_V] := ViewSy;
   PibTerm_Command_Table[Alt_W] := HostSy;
   PibTerm_Command_Table[Alt_X] := QuitSy;
   PibTerm_Command_Table[Alt_Y] := TimersSy;
   PibTerm_Command_Table[Alt_Z] := AreaCodeSy;

   PibTerm_Command_Table[Shift_Tab] := AddLfSy;

   FOR I := F1 TO F10 DO
      PibTerm_Command_Table[I] := KeySendSy;

   FOR I := Alt_F1 TO Alt_F10 DO
      PibTerm_Command_Table[I] := KeySendSy;

   FOR I := Shift_F1 TO Shift_F10 DO
      PibTerm_Command_Table[I] := KeySendSy;

   FOR I := Ctrl_F1 TO Ctrl_F10 DO
      PibTerm_Command_Table[I] := KeySendSy;

   PibTerm_Command_Table[U_Arrow]      := KeySendSy;
   PibTerm_Command_Table[Alt_U_Arrow]  := KeySendSy;
   PibTerm_Command_Table[Ctrl_U_Arrow] := KeySendSy;

   PibTerm_Command_Table[L_Arrow]      := KeySendSy;
   PibTerm_Command_Table[Alt_L_Arrow]  := KeySendSy;
   PibTerm_Command_Table[Ctrl_L_Arrow] := KeySendSy;

   PibTerm_Command_Table[R_Arrow]      := KeySendSy;
   PibTerm_Command_Table[Alt_R_Arrow]  := KeySendSy;
   PibTerm_Command_Table[Ctrl_R_Arrow] := KeySendSy;

   PibTerm_Command_Table[D_Arrow]      := KeySendSy;
   PibTerm_Command_Table[Alt_D_Arrow]  := KeySendSy;
   PibTerm_Command_Table[Ctrl_D_Arrow] := KeySendSy;

   PibTerm_Command_Table[Home]         := KeySendSy;
   PibTerm_Command_Table[Alt_Home]     := KeySendSy;

   PibTerm_Command_Table[PgUp]         := KeySendSy;
   PibTerm_Command_Table[Alt_PgUp]     := KeySendSy;
   PibTerm_Command_Table[Ctrl_PgUp]    := KeySendSy;

   PibTerm_Command_Table[End_Key]      := KeySendSy;
   PibTerm_Command_Table[Alt_End_Key]  := KeySendSy;
   PibTerm_Command_Table[Ctrl_End_Key] := KeySendSy;

   PibTerm_Command_Table[PgDn]         := KeySendSy;
   PibTerm_Command_Table[Alt_PgDn]     := KeySendSy;
   PibTerm_Command_Table[Ctrl_PgDn]    := KeySendSy;

   PibTerm_Command_Table[Ins_Key]      := KeySendSy;
   PibTerm_Command_Table[Alt_Ins_Key]  := KeySendSy;
   PibTerm_Command_Table[Ctrl_Ins_Key] := KeySendSy;

   PibTerm_Command_Table[Del_Key]      := KeySendSy;
   PibTerm_Command_Table[Alt_Del_Key]  := KeySendSy;
   PibTerm_Command_Table[Ctrl_Del_Key] := KeySendSy;

                                   (* Initialize Kermit variables *)

                                   (* Timeout value *)
   Kermit_Timeout      := 5;
                                   (* Default checksum method *)
   Kermit_Chk_Type     := '1';
                                   (* Marks repeated data *)
   Kermit_Repeat_Char  := '~';
                                   (* But repeating not implemented yet *)
   Repeating           := FALSE;
                                   (* Time to wait before send in host mode *)
   Kermit_Delay_Time   := 15;
                                   (* Initial packet size *)

   Kermit_Packet_Size  := Kermit_Init_Packet_Size;

                                   (* Padding not required *)
   Kermit_Npad         := 0;
   Kermit_Pad_Char     := CHR( 0 );

                                   (* Don't send padding unless requested *)
   My_Pad_Num          := 0;
                                   (* CR is initial EOL *)

   Kermit_EOL          := CHR( CR );

                                   (* Send a CR to start with *)
   Send_EOL            := CR;
                                   (* Use the default quote *)
   Kermit_Quote_Char   := '#';
                                   (* Use the default 8 bit quote *)
   Kermit_Quote_8_Char := '&';
                                   (* But don't ask for it unless binary with parity *)
   Quoting             := FALSE;
                                   (* No debugging in Kermit *)
   Kermit_Debug        := FALSE;
                                   (* SOH is default header character *)
   Kermit_Header_Char  := CHR( SOH );

END   (* Set_Defaults *);
